import os

from aisuite4cn.base_provider import BaseProvider


class QwenProvider(BaseProvider):
    """
    Aliyun Qwen Provider
    """

    def __init__(self, **config):
        """
        Initialize the Qwen shot provider with the given configuration.
        Pass the entire configuration dictionary to the Qwen client constructor.
        """
        # Ensure API key is provided either in config or via environment variable
        current_config = dict(config)
        current_config.setdefault("api_key", os.getenv("DASHSCOPE_API_KEY"))
        if not current_config['api_key']:
            raise ValueError(
                "Dashscope API key is missing. Please provide it in the config or set the DASHSCOPE_API_KEY environment variable."
            )
        super().__init__('https://dashscope.aliyuncs.com/compatible-mode/v1',
                         **current_config)



import os

from aisuite4cn import BaseProvider


class ArkProvider(BaseProvider):
    """
    ByteDance Ark Provider
    """

    def __init__(self, **config):
        """
        Initialize the Volcengine provider with the given configuration.
        Pass the entire configuration dictionary to the Volcengine client constructor.
        """
        # Ensure API key is provided either in config or via environment variable
        current_config = dict(config)
        current_config.setdefault("api_key", os.getenv("ARK_API_KEY"))
        if not current_config["api_key"]:
            raise ValueError(
                "Ark API key is missing. Please provide it in the config or set the ARK_API_KEY environment variable."
            )

        super().__init__('https://ark.cn-beijing.volces.com/api/v3',
                         **current_config)



import os

from aisuite4cn.base_provider import BaseProvider


class DmxapiProvider(BaseProvider):
    """
    A provider for the DMXAPI API.
    """

    def __init__(self, **config):
        """
        Initialize the Dmxapi provider with the given configuration.
        Pass the entire configuration dictionary to the Dmxapi client constructor.
        """

        current_config = dict(config)
        current_config.setdefault("api_key", os.getenv("DMXAPI_API_KEY", None))
        if not current_config['api_key']:
            raise ValueError(
                "Dmxapi API key is missing. Please provide it in the config or set the DMXAPI_API_KEY environment variable."
            )
        base_url = current_config.pop("base_url", os.getenv("DMXAPI_BASE_URL", 'https://www.dmxapi.cn/v1/'))
        super().__init__(base_url, **current_config)



import os
import time
from typing import Optional

import openai
from pydantic import BaseModel
from qianfan.resources.console.iam import IAM

from aisuite4cn import BaseProvider


class BearerToken(BaseModel):
    user_id: Optional[str] = None
    token: Optional[str] = None
    status: Optional[str] = None
    create_time: float = 0
    expire_time: float = 0


class QianfanProvider(BaseProvider):
    """
    Baidu Qianfan Provider

    ref: https://cloud.baidu.com/doc/WENXINWORKSHOP/s/em4tsqo3v
    """

    access_key: Optional[str] = None
    secret_key: Optional[str] = None
    bearerToken: Optional[BearerToken] = None

    def __init__(self, **config):
        """
        Initialize the Qianfan provider with the given configuration.
        Pass the entire configuration dictionary to the Qianfan client constructor.
        """
        # Ensure access key and secret key is provided either in config or via environment variable

        self.config = dict(config)

        self.config.setdefault("api_key", os.getenv("QIANFAN_API_KEY"))

        if self.config['api_key']:
            self.use_api_key = True
            self.config.pop("access_key")
            self.config.pop("secret_key")
        else:
            self.use_api_key = False
            self.access_key = self.config.pop("access_key", os.getenv("QIANFAN_ACCESS_KEY"))
            self.secret_key = self.config.pop("secret_key", os.getenv("QIANFAN_SECRET_KEY"))
            if not self.access_key:
                raise ValueError(
                    "Qainfan access key is missing. Please provide it in the config or set the QIANFAN_ACCESS_KEY environment variable."
                )
            if not self.secret_key:
                raise ValueError(
                    "Qianfan secret key is missing. Please provide it in the config or set the QIANFAN_SECRET_KEY environment variable."
                )

        super().__init__(
            base_url='https://qianfan.baidubce.com/v2',
            **self.config)

    @property
    def client(self):
        """Getter for the OpenAI client."""
        if not self._client:
            if self.use_api_key:
                self._client = openai.OpenAI(base_url=self.base_url, **self.config)
            else:
                self._client = openai.OpenAI(
                    api_key=self.get_bearer_token(),
                    base_url=self.base_url,
                    **self.config)
            return self._client

        if not self.use_api_key:
            self._client.api_key = self.get_bearer_token()
        return self._client

    @property
    def async_client(self):
        """Getter for the asynchronous OpenAI client.

        Lazily initializes the AsyncOpenAI client if not already created.
        """
        if not self._async_client:
            if self.use_api_key:
                self._async_client = openai.AsyncOpenAI(
                    base_url=self.base_url,
                    **self.config
                )
            else:
                self._async_client = openai.AsyncOpenAI(
                    api_key=self.get_bearer_token(),
                    base_url=self.base_url,
                    **self.config
                )
            return self._async_client

        if not self.use_api_key:
            self._async_client.api_key = self.get_bearer_token()
        return self._async_client

    def get_bearer_token(self):
        if self.bearerToken is None:
            self.bearerToken = BearerToken()
        if time.time() < self.bearerToken.expire_time:
            return self.bearerToken.token

        os.environ["QIANFAN_ACCESS_KEY"] = self.access_key
        os.environ["QIANFAN_SECRET_KEY"] = self.secret_key
        expire_in_seconds = 86400
        response = IAM.create_bearer_token(expire_in_seconds=expire_in_seconds)
        self.bearerToken.user_id = response.body.get('user_id')
        self.bearerToken.token = response.body.get('token')
        self.bearerToken.status = response.body.get('enable')
        self.bearerToken.create_time = time.time()
        self.bearerToken.expire_time = self.bearerToken.create_time + expire_in_seconds - 120
        return self.bearerToken.token



import os

from aisuite4cn.base_provider import BaseProvider


class SiliconflowProvider(BaseProvider):
    """
    Siliconflow Provider
    """

    def __init__(self, **config):
        """
        Initialize the Siliconflow provider with the given configuration.
        Pass the entire configuration dictionary to the Siliconflow client constructor.
        """
        # Ensure API key is provided either in config or via environment variable

        current_config = dict(config)
        current_config.setdefault("api_key", os.getenv("SILICONFLOW_API_KEY"))
        if not current_config['api_key']:
            raise ValueError(
                "Siliconflow API key is missing. Please provide it in the config or set the SILICONFLOW_API_KEY environment variable."
            )

        super().__init__(
            base_url='https://api.siliconflow.cn/v1',
            **current_config)



import os

from aisuite4cn.base_provider import BaseProvider


class ZhipuaiProvider(BaseProvider):
    """
    Zhipu Provider
    """

    def __init__(self, **config):
        """
        Initialize the Zhipu provider with the given configuration.
        Pass the entire configuration dictionary to the Zhipu client constructor.
        """
        # Ensure API key is provided either in config or via environment variable

        current_config = dict(config)
        current_config.setdefault("api_key", os.getenv("ZHIPUAI_API_KEY"))
        if not current_config["api_key"]:
            raise ValueError(
                "Zhipu API key is missing. Please provide it in the config or set the ZHIPUAI_API_KEY environment variable."
            )

        super().__init__('https://open.bigmodel.cn/api/paas/v4',
                         **current_config)

    def _prepare_chat_completions_call(self, model, messages, **kwargs):
        new_kwargs = dict(kwargs)
        # Note: Zhipu does not support the frequency_penalty and presence_penalty parameters.
        new_kwargs.pop('frequency_penalty', None)
        new_kwargs.pop('presence_penalty', None)
        return model, messages, new_kwargs




import os
from urllib.parse import parse_qs

from aisuite4cn.base_provider import BaseProvider


class SparkProvider(BaseProvider):
    """
    Spark Provider
    """

    def __init__(self, **config):
        """
        Initialize the Spark provider with the given configuration.
        Pass the entire configuration dictionary to the Spark client constructor.
        api_key_map Example:
        {
            "api_key_map": {
                # Key is the model name, and value is the corresponding API password.
                # This mapping allows you to specify different API passwords for different models.
                # Example:
                "4.0Ultra":"your-4.0Ultra-APIPassword", # API password for the 4.0Ultra model
                "generalv3":"your-generalv3-APIPassword" # API password for the generalv3 model
            }
        }
        """
        # Ensure API key is provided either in config or via environment variable

        current_config = dict(config)
        current_config.setdefault("api_key", "default")
        env_api_key_map = {k: v[0] for k, v in parse_qs(os.getenv("SPARK_API_KEY_MAP", "")).items()}
        self.api_key_map = current_config.pop("api_key_map", env_api_key_map)

        super().__init__('https://spark-api-open.xf-yun.com/v1',
                         **current_config)

    def _get_api_key_from_api_key_map(self, model):
        if not self.api_key_map[model]:
            raise ValueError(
                "Spark API key is missing. Please provide it in the config or set the SPARK_API_KEY_MAP environment variable."
            )
        return self.api_key_map[model]

    def _prepare_chat_completions_call(self, model, messages, **kwargs):
        self.client.api_key = self._get_api_key_from_api_key_map(model)
        self.async_client.api_key = self._get_api_key_from_api_key_map(model)
        return model, messages, kwargs



import os

from aisuite4cn.base_provider import BaseProvider


class HunyuanProvider(BaseProvider):
    """
    Tecent Hunyuan Provider

    API refer to: https://cloud.tencent.com/document/product/1729/111007
    """

    def __init__(self, **config):
        """
        Initialize the Hunyuan provider with the given configuration.
        Pass the entire configuration dictionary to the Hunyuan client constructor.
        """
        # Ensure access key and secret key is provided either in config or via environment variable

        current_config = dict(config)
        current_config.setdefault("api_key", os.getenv("HUNYUAN_API_KEY"))
        if not current_config['api_key']:
            raise ValueError(
                "Hunyuan api key is missing. Please provide it in the config or set the HUNYUAN_API_KEY environment variable."
            )

        super().__init__('https://api.hunyuan.cloud.tencent.com/v1',
                         **current_config)



import os

from aisuite4cn.base_provider import BaseProvider


class OllamaProvider(BaseProvider):

    def __init__(self, **config):
        """
        Initialize the Ollama provider with the given configuration.
        Pass the entire configuration dictionary to the Ollama client constructor.
        """

        current_config = dict(config)

        base_url = current_config.pop("base_url", os.getenv("OLLAMA_BASE_URL"))
        if not base_url:
            raise ValueError(
                "Ollama Base Url is missing. Please provide it in the config or set the OLLAMA_BASE_URL environment variable."
            )
        current_config['api_key'] = current_config.get('api_key', os.getenv("OLLAMA_API_KEY",  "ollama"))
        # Pass the entire config to the Ollama client constructor
        super().__init__(base_url,
                         **current_config)



from abc import ABC

from .provider import ProviderFactory


def _get_provider_key_and_model_name(model) -> tuple[str, str]:
    """
    Extract the provider key and model name from the model string.
    :param model: model, example: 'provider:model'
    :return: provider_key, model_name
    """
    if ":" not in model:
        raise ValueError(
            f"Invalid model format. Expected 'provider:model', got '{model}'"
        )
    # Find the first ':' to split provider_key and model_name
    separator_index = model.find(':')
    if separator_index == -1:
        raise ValueError("Model identifier must contain a ':' to specify the provider key and model name.")
    model_name = model[separator_index + 1:]
    provider_key = model[:separator_index]
    return provider_key, model_name


class Client:
    def __init__(self, provider_configs=None):
        """
        Initialize the client with provider configurations.
        Use the ProviderFactory to create provider instances.

        Args:
            provider_configs (dict): A dictionary containing provider configurations.
                Each key should be a provider string (e.g., "google" or "aws-bedrock"),
                and the value should be a dictionary of configuration options for that provider.
                For example:
                {
                    "openai": {"api_key": "your_openai_api_key"},
                    "aws-bedrock": {
                        "aws_access_key": "your_aws_access_key",
                        "aws_secret_key": "your_aws_secret_key",
                        "aws_region": "us-west-2"
                    }
                }
        """
        if provider_configs is None:
            provider_configs = {}
        self.providers = {}
        self.provider_configs = provider_configs
        self._chat = None
        self._embeddings = None
        self._initialize_providers()

    def _initialize_providers(self):
        """Helper method to initialize or update providers."""
        for provider_key, config in self.provider_configs.items():
            provider_key = self._validate_provider_key(provider_key)
            self.providers[provider_key] = ProviderFactory.create_provider(
                provider_key, config
            )

    def _validate_provider_key(self, provider_key):
        """
        Validate if the provider key corresponds to a supported provider.
        """
        supported_providers = ProviderFactory.get_supported_providers()

        if provider_key not in supported_providers:
            raise ValueError(
                f"Invalid provider key '{provider_key}'. Supported providers: {supported_providers}. "
                "Make sure the model string is formatted correctly as 'provider:model'."
            )

        return provider_key

    def configure(self, provider_configs: dict = None):
        """
        Configure the client with provider configurations.
        """
        if provider_configs is None:
            return

        self.provider_configs.update(provider_configs)
        self._initialize_providers()  # NOTE: This will override existing provider instances.

    @property
    def chat(self):
        """Return the chat API interface."""
        if not self._chat:
            self._chat = Chat(self)
        return self._chat


    @property
    def embeddings(self):
        """Return the embeddings API interface."""
        if not self._embeddings:
            self._embeddings = Embeddings(self)
        return self._embeddings


class AsyncClient(Client):

    @property
    def chat(self):
        """Return the chat API interface."""
        if not self._chat:
            self._chat = AsyncChat(self)
        return self._chat

    @property
    def embeddings(self):
        """Return the embeddings API interface."""
        if not self._embeddings:
            self._embeddings = AsyncEmbeddings(self)
        return self._embeddings


class Chat:
    def __init__(self, client: "Client"):
        self.client = client
        self._completions = Completions(self.client)

    @property
    def completions(self):
        """Return the completions interface."""
        return self._completions


class AsyncChat:
    def __init__(self, client: "AsyncClient"):
        self.client = client
        self._completions = AsyncCompletions(self.client)

    @property
    def completions(self):
        """Return the completions interface."""
        return self._completions


class BaseCommon(ABC):

    def __init__(self, client):
        self.client = client

    @staticmethod
    def _get_provider_key_and_model_name(model):
        """
        Extract the provider key and model name from the model string.
        """
        if ":" not in model:
            raise ValueError(
                f"Invalid model format. Expected 'provider:model', got '{model}'"
            )
        # Find the first ':' to split provider_key and model_name
        separator_index = model.find(':')
        if separator_index == -1:
            raise ValueError("Model identifier must contain a ':' to specify the provider key and model name.")
        model_name = model[separator_index + 1:]
        provider_key = model[:separator_index]
        return provider_key, model_name

    def _get_provider(self, provider_key):
        # Validate if the provider is supported
        supported_providers = ProviderFactory.get_supported_providers()
        if provider_key not in supported_providers:
            raise ValueError(
                f"Invalid provider key '{provider_key}'. Supported providers: {supported_providers}. "
                "Make sure the model string is formatted correctly as 'provider:model'."
            )
        # Initialize provider if not already initialized
        if provider_key not in self.client.providers:
            config = self.client.provider_configs.get(provider_key, {})
            self.client.providers[provider_key] = ProviderFactory.create_provider(
                provider_key, config
            )
        provider = self.client.providers.get(provider_key)
        if not provider:
            raise ValueError(f"Could not load provider for '{provider_key}'.")
        return provider


class AsyncCompletions(BaseCommon):

    def __init__(self, client: "AsyncClient"):
        super().__init__(client)

    async def create(self, model: str, messages: list, **kwargs):
        """
        Create chat completion based on the model, messages, and any extra arguments.
        """
        # Check that correct format is used
        provider_key, model_name = _get_provider_key_and_model_name(model)

        provider = self._get_provider(provider_key)

        # Delegate the chat completion to the correct provider's implementation
        return await provider.async_chat_completions_create(model_name, messages, **kwargs)


    async def parse(self, model, messages, **kwargs):
        """
        Parse chat completion based on the model, messages, and any extra arguments.
        """
        # Check that correct format is used
        provider_key, model_name = _get_provider_key_and_model_name(model)

        provider = self._get_provider(provider_key)

        # Delegate the chat completion to the correct provider's implementation
        return await provider.async_chat_completions_parse(model_name, messages, **kwargs)

    def async_chat_completions_stream(self, model, messages, **kwargs):
        """
        Stream chat completion based on the model, messages, and any extra arguments.
        """
        # Check that correct format is used
        provider_key, model_name = _get_provider_key_and_model_name(model)

        provider = self._get_provider(provider_key)

        # Delegate the chat completion to the correct provider's implementation
        return provider.async_chat_completions_stream(model_name, messages, **kwargs)


class Completions(BaseCommon):

    def __init__(self, client: "Client"):
        super().__init__(client)

    def create(self, model: str, messages: list, **kwargs):
        """
        Create chat completion based on the model, messages, and any extra arguments.
        """
        # Check that correct format is used
        provider_key, model_name = _get_provider_key_and_model_name(model)

        provider = self._get_provider(provider_key)

        # Delegate the chat completion to the correct provider's implementation
        return provider.chat_completions_create(model_name, messages, **kwargs)


    def parse(self, model, messages, **kwargs):
        """
        Parse chat completion based on the model, messages, and any extra arguments.
        """
        # Check that correct format is used
        provider_key, model_name = _get_provider_key_and_model_name(model)

        provider = self._get_provider(provider_key)

        # Delegate the chat completion to the correct provider's implementation
        return provider.chat_completions_parse(model_name, messages, **kwargs)

    def chat_completions_stream(self, model, messages, **kwargs):
        """
        Stream chat completion based on the model, messages, and any extra arguments.
        """
        # Check that correct format is used
        provider_key, model_name = _get_provider_key_and_model_name(model)

        provider = self._get_provider(provider_key)

        # Delegate the chat completion to the correct provider's implementation
        return provider.chat_completions_stream(model_name, messages, **kwargs)




class Embeddings(BaseCommon):
    def __init__(self, client: "Client"):
        super().__init__(client)

    def create(self, model: str, input, **kwargs):
        # Check that correct format is used
        provider_key, model_name = _get_provider_key_and_model_name(model)

        provider = self._get_provider(provider_key)
        return provider.embeddings_create(model_name, input, **kwargs)




class AsyncEmbeddings(BaseCommon):
    def __init__(self, client: "AsyncClient"):
        super().__init__(client)

    async def create(self, model: str, input, **kwargs):
        # Check that correct format is used
        provider_key, model_name = _get_provider_key_and_model_name(model)

        provider = self._get_provider(provider_key)
        return await provider.async_embeddings_create(model_name, input, **kwargs)





import openai
from aisuite4cn.provider import Provider

OPENAI_PARAMS = [
    "messages",
    "model",
    "audio",
    "response_format",
    "frequency_penalty",
    "function_call",
    "functions",
    "logit_bias",
    "logprobs",
    "max_completion_tokens",
    "max_tokens",
    "metadata",
    "modalities",
    "n",
    "parallel_tool_calls",
    "prediction",
    "presence_penalty",
    "prompt_cache_key",
    "reasoning_effort",
    "safety_identifier",
    "seed",
    "service_tier",
    "stop",
    "store",
    "stream",
    "stream_options",
    "temperature",
    "tool_choice",
    "tools",
    "top_logprobs",
    "top_p",
    "user",
    "verbosity"
]


class BaseProvider(Provider):
    """Base class for all openai compatible providers."""

    def __init__(self, base_url, **config):
        self.base_url = base_url
        self.config = dict(config)
        self._client = None
        self._async_client = None

    @property
    def client(self):
        """Getter for the OpenAI client."""
        if not self._client:
            self._client = openai.OpenAI(base_url=self.base_url, **self.config)
        return self._client

    @client.setter
    def client(self, value):
        """Setter for the OpenAI client."""
        self._client = value

    @property
    def async_client(self):
        """Getter for the asynchronous OpenAI client.

        Lazily initializes the AsyncOpenAI client if not already created.
        """
        if not self._async_client:
            self._async_client = openai.AsyncOpenAI(
                base_url=self.base_url,
                **self.config
            )
        return self._async_client

    @async_client.setter
    def async_client(self, value):
        """Setter for the asynchronous OpenAI client.

        Allows replacing the default client with a custom one.

        Args:
            value: An instance of openai.AsyncOpenAI or compatible client
        """
        self._async_client = value

    def _prepare_chat_completions_call(self, model, messages, **kwargs):
        return model, messages, kwargs

    def chat_completions_create(self, model, messages, **kwargs):
        """Create a chat completion using the OpenAI API."""
        model, messages, new_kwargs = self._prepare_chat_completions_call(model, messages, **kwargs)
        return self.client.chat.completions.create(
            model=model,
            messages=messages,
            **self._compatible_with_openai_kwargs(new_kwargs)
        )


    def chat_completions_parse(self, model, messages, **kwargs):
        model, messages, new_kwargs = self._prepare_chat_completions_call(model, messages, **kwargs)
        return self.client.chat.completions.parse(
            model=model,
            messages=messages,
            **self._compatible_with_openai_kwargs(new_kwargs)
        )


    def chat_completions_stream(self, model, messages, **kwargs):
        model, messages, new_kwargs = self._prepare_chat_completions_call(model, messages, **kwargs)
        return self.client.chat.completions.stream(
            model=model,
            messages=messages,
            **self._compatible_with_openai_kwargs(new_kwargs)
        )


    async def async_chat_completions_create(self, model, messages, **kwargs):
        """Create a chat completion using the OpenAI API."""
        model, messages, new_kwargs = self._prepare_chat_completions_call(model, messages, **kwargs)
        return await self.async_client.chat.completions.create(
            model=model,
            messages=messages,
            **self._compatible_with_openai_kwargs(new_kwargs)
        )

    async def async_chat_completions_parse(self, model, messages, **kwargs):

        model, messages, new_kwargs = self._prepare_chat_completions_call(model, messages, **kwargs)
        return await self.async_client.chat.completions.parse(
            model=model,
            messages=messages,
            **self._compatible_with_openai_kwargs(new_kwargs)
        )

    def async_chat_completions_stream(self, model, messages, **kwargs):
        model, messages, new_kwargs = self._prepare_chat_completions_call(model, messages, **kwargs)
        return self.async_client.chat.completions.stream(
            model=model,
            messages=messages,
            **self._compatible_with_openai_kwargs(new_kwargs)
        )

    def embeddings_create(self, model, input, **kwargs):
        return self.client.embeddings.create(
            model=model,
            input=input,
            **kwargs
        )

    async def async_embeddings_create(self, model, input, **kwargs):
        return await self.async_client.embeddings.create(
            model=model,
            input=input,
            **kwargs
        )

    @staticmethod
    def _compatible_with_openai_kwargs(kwargs: dict = None) -> dict:
        # add logic to convert kwargs to openai compatible kwargs
        new_kwargs = dict(kwargs) if kwargs else {}
        new_extra_body = dict(kwargs.get("extra_body", {}))
        for k, v in kwargs.items():
            if k not in OPENAI_PARAMS:
                new_extra_body[k] = new_kwargs.pop(k)
        if new_extra_body:
            new_kwargs["extra_body"] = new_extra_body
        return new_kwargs



import openai
import os
from aisuite4cn.base_provider import BaseProvider


class MoonshotProvider(BaseProvider):
    """
    Moonshot Provider
    """
    def __init__(self, **config):
        """
        Initialize the Moonshot provider with the given configuration.
        Pass the entire configuration dictionary to the Moonshot client constructor.
        """
        # Ensure API key is provided either in config or via environment variable

        current_config = dict(config)
        current_config.setdefault("api_key", os.getenv("MOONSHOT_API_KEY"))
        if not current_config['api_key']:
            raise ValueError(
                "Moonshot API key is missing. Please provide it in the config or set the MOONSHOT_API_KEY environment variable."
            )

        super().__init__('https://api.moonshot.cn/v1',
                         **current_config)

